#!/usr/bin/ruby
# coding: utf-8

include Math
require "narray"

#******************************************************************************
# subroutine solvekepler
#------------------------------------------------------------------------------
# This subroutine solves Kepler's equation and is used to obtain eccentric
# anomaly. 
# Calculation method of these values are based on those described in
# Duffett-Smith [1988].
# Reference
#   Peter Duffett-Smith, Practical Astronomy with Your Calculator Third
#   Edition, Cambridge University Press, pp. 72-73, 1988
#==============================================================================
# input parameters
#------------------------------------------------------------------------------
# eccen    : Eccentricity
# manom    : Mean anomaly (radian)
#------------------------------------------------------------------------------
# output parameters
#------------------------------------------------------------------------------
# eanom    : Eccentric anomaly (radian)
#******************************************************************************

def solve_Kepler( sEccen, sMAnom )

  sAccuracy = 1.0e-6

  sEAnom = sMAnom
  for l in 1..1000
    sEAnomError = sEAnom - sEccen * sin( sEAnom ) - sMAnom

    break if sEAnomError.abs < sAccuracy

    sEAnom = sEAnom - sEAnomError / ( 1.0 - sEccen * sin( sEAnom ) )
    sEAnom = sEAnom % ( 2 * PI )
  end
  if l >= 1000 then
    print "Cannot find a solution to Kepler's equation\n"
    exit
  end

  return sEAnom
end

# sSMAxis : semi-major axis
# sEccen  : eccentricity
# sPerLon : perihelion longitude
# sSecInAYear : seconds in a year (tropical year in seconds)
# sEpsOrb : inclination of equator of orbit
# sPerLs  : perihelion longitude from vernal equinox
# sMLonAtEpoch : mean longitude at epoch
# sSecFromEpoch : seconds from epoch

def planetorbit( sSMAxis, sEccen, sPerLon, sSecInAYear, sEpsOrb, sPerLs, sMLonAtEpoch, sSecFromEpoch )


  #     d        : the number of days since the epoch 1990.0


  #     manom    : Mean anomaly (degree)
  #     eanom    : eccentric anomaly (degree)
  #     tanom    : True anomaly (degree)
  #     arglat   : argument of ecliptic latitude

  #     x0,y0,z0 : Temporary variables for calculation of solar declination angle
  #     x1,y1,z1 : Temporary variables for calculation of solar declination angle

  sMAnom = 360.0 * sSecFromEpoch / sSecInAYear + sMLonAtEpoch - sPerLon

  #     This formulation was obtained from "Tenmon Nenkan".
  #      d=julday-2451545.0d0
  #      manom=355.4330d0+0.52407108*d+0.003d0*d/36525.0d0*d/36525.0d0
  #     &     -(336.0602d0+1.8410d0*d/36525.0d0
  #     &     +0.0001d0*d/36525.0d0*d/36525.0d0)

  sMAnom = sMAnom % 360.0

  sMAnom *= PI / 180.0

  sEAnom = solve_Kepler( sEccen, sMAnom )

  sTAnom = sqrt( (1.0+sEccen)/(1.0-sEccen) ) * tan( sEAnom / 2.0 )
  sTAnom = atan( sTAnom )
  sTAnom = sTAnom * 2.0
  sTAnom = sTAnom % ( 2 * PI )

  sHelDis = sSMAxis * ( 1.0 - sEccen**2 ) / ( 1.0 + sEccen * cos( sTAnom ) )

  sLs = sTAnom + sPerLs * PI/180.0
  sLs = sLs % ( 2 * PI )

  sSinDel = sin( sEpsOrb * PI / 180.0 ) * sin( sLs )

  return sHelDis, sSinDel, sLs, sTAnom
end

def planetorbitWithLonLat( sSMAxis, sEccen, sPerLon, sSecInAYear, sEpsOrb, sPerLs, sMLonAtEpoch, sSecFromEpoch, sOrbInc, sAscLon )

  sHelDis, sSinDel, sLs, sTAnom = planetorbit( sSMAxis, sEccen, sPerLon, sSecInAYear, sEpsOrb, sPerLs, sMLonAtEpoch, sSecFromEpoch )


  sArgLat = sPerLon + sTAnom * 180.0/PI- sAscLon
  sArgLat = sArgLat % 360.0

  sEclLon = sAscLon + atan( cos( sOrbInc * PI/180.0 ) * tan( sArgLat * PI/180.0 ) ) * 180.0/PI
  if ( sArgLat > 90.0 ) && ( sArgLat <= 180.0 ) then
    sEclLon = sEclLon - (-90.0) + 90.0
  elsif ( sArgLat > 180.0 ) && ( sArgLat <= 270.0 ) then
    sEclLon = sEclLon + 180.0
  end
  sEclLon = sEclLon % 360.0

  sEclLat = asin(sin( sOrbInc * PI/180.0 ) * sin( sArgLat * PI/180.0 ) ) * 180.0/PI

  return sHelDis, sSinDel, sLs, sEclLon, sEclLat
end

#******************************************************************************
#     subroutine actojd
#     calculate julian day from AC
#     If AC (yyyy,mm,dd) is given, julian day will be calculated.
#
#     Peter Duffett-Smit, Practical Astronomy with Your Calculator
#     Third Edition, p 7, Cambridge University Press, 2000
#******************************************************************************
#     yyyy      : year
#     mm        : month
#     dd        : day
#     julday    : julian day
#******************************************************************************

def ac2jd( yyyy, mm, dd )

  y = yyyy
  m = mm
  d = dd

  if (mm == 1) || (mm ==2)  then
    y = (y-1).to_f
    m = m + 12.0
  end

  if yyyy > 1582  then
    tmpa = (y/100.0).to_i
    tmpb = 2.0 - tmpa + (tmpa/4.0).to_i
  elsif yyyy >= 1582 then
    print "Year 1582 cannot be treated in this program.\n"
    exit
  else
    tmpb = 0.0
  end

  if y < 0.0 then
    tmpc = ( (365.25*y)-0.75 ).to_i
  else
    tmpc = (365.25*y).to_i
  end

  tmpd = ( 30.6001*(m+1) ).to_i

  julday = tmpb + tmpc + tmpd + d +1720994.5

  return julday
end


def ty_insolation( sNOrbitDiv, y_Lat, sSMAxis, sEccen, sPerLon, sSecInAYear, sEpsOrb, sPerLs, sMLonAtEpoch )

  imax      =  24
  jmax      =  y_Lat.size

  a_Time  = NArray.float(sNOrbitDiv)
  a_Ls    = NArray.float(sNOrbitDiv)
  ay_Ins  = NArray.float(sNOrbitDiv,jmax).fill(0.0)

  y_SinLat = NMath.sin( y_Lat * PI/180.0 )
  y_CosLat = NMath.cos( y_Lat * PI/180.0 )


  # ep : the epoch 1990.0, epoch 1990.0 probably means 1990/01/00.
  #sEpoch1990 = 2447891.50
  #
  sYYYY = 1990
  sMM   =    1
  sDD   =    0
  sJDEpoch1990 = ac2jd( sYYYY, sMM, sDD )

  #print "Input date (YYYY/MM/DD): "
  #sDate = gets.split("/")
#  sYYYY = 2001
#  sMM   =    1
#  sDD   =    1
#  sJD0  = ac2jd( sYYYY, sMM, sDD )

  sJD0  = sJDEpoch1990

  for l in 0..(sNOrbitDiv-1)
    sJD = sJD0 + sSecInAYear / ( 60*60*24 ) / (sNOrbitDiv-1) * l
    sSecFromEpoch = ( sJD - sJDEpoch1990 ) * ( 60*60*24 )

    sHeldis, sSinDel, sLs = planetorbit( sSMAxis, sEccen, sPerLon, sSecInAYear, sEpsOrb, sPerLs, sMLonAtEpoch, sSecFromEpoch )

    a_Time[l] = sJD - sJD0
    a_Ls  [l] = sLs

    for i in 0..(imax-1)
      sHourAngle = 360.0 / imax * i
      y_CosZet = y_SinLat * sSinDel + y_CosLat * sqrt( 1.0 - sSinDel**2 ) * cos( sHourAngle )
      y_CosZet[ y_CosZet.lt 0.0 ] = 0.0
      ay_Ins[l,true] += y_CosZet
    end
    ay_Ins[l,true] /= imax
  end

  return a_Time, ay_Ins
end
